#include "glassexe.h"

void printUsage(ostream& out) 
{
  out << "Usage:    glass <hfile> <mfile> [options]\n\n";
  out << "Options:\n\n";
  out << "\t-o <filename>\n";
  out << "\t\tPrint output to <filename> instead of STDOUT\n";
  out << "\t-c\n";
  out << "\t\tPrint Clustal W header in output\n";
  out << "\t-r\n";
  out << "\t\tOutput only aligning regions\n";
  out << endl;
}


void parseFromCmdLine(int argc, char* argv[], GlassExeRegistry& registry) 
{
  // Parse the optional arguments.
  int c = 0;
  while((c = getopt(argc, argv, "o:cr")) != -1) {
    switch(c) {
    case 'o':
      registry.outputFileName = optarg;
      break;
    case 'c':
      registry.useClustalHeader = true;
      break;
    case 'r':
      registry.outputRegions = true;
      break;
    case '?':
      cerr << "Unknown option -" << optopt << "." << endl;
      exit(EXIT_FAILURE);
    default:
      abort();
    }
  }
  // Parse the required arguments.
  if(optind + 2 == argc) {
    registry.humanFileName = string(argv[optind]);
    registry.mouseFileName = string(argv[optind+1]);
  }
  else {
    printUsage(cerr);
    exit(EXIT_FAILURE);
  }
}


void outputResult(ostream& out, vector<PairwiseAlignment>& img,
		  GlassExeRegistry& registry) {
  if(registry.useClustalHeader) {
      out << registry.clustalHeader;
  }
  else {
      out << registry.glassHeader;
  }
  if(!registry.outputRegions) {
    PairwiseAlignmentIO::writeClustal(out, img[0]);
  }
  else {
    PairwiseAlignmentIO::writeRegions(out, img);
  }
}


int main(int argc, char* argv[]) 
{
  GlassExeRegistry registry;
  parseFromCmdLine(argc, argv, registry);

  ifstream hfile(registry.humanFileName.data());
  ifstream mfile(registry.mouseFileName.data());

  Sequence hseq = SequenceIO::readFasta(hfile);
  Sequence mseq = SequenceIO::readFasta(mfile);  

  Glass glass;
  // ifstream regfile("/d/tarjei/wgc/rosetta/glass.registry"); (Disabled since 1.1a)
  // glass.setParamsFromFile(regfile);
  vector<PairwiseAlignment> img;
  if(registry.outputRegions) {
    img = glass.alignRegions(hseq, mseq);
  }
  else {
    img.push_back(glass.align(hseq, mseq));
  }

  if(registry.outputFileName != "") {
    ofstream fout(registry.outputFileName.c_str());
    outputResult(fout, img, registry);
  }
  else {
    outputResult(cout, img, registry);
  }

  exit(EXIT_SUCCESS);
}

